@@ -37,7 +37,7 @@
 #include <QStandardPaths>
 #include <QCollator>
 #include <QSysInfo>
-
+#include <qrandom.h>
 
 #ifdef Q_OS_UNIX
 #include <sys/statvfs.h>
@@ -49,13 +49,13 @@
 #include <cstdarg>
 #include <cstring>
 
-#if defined(Q_OS_WIN)
-#include "utility_win.cpp"
-#elif defined(Q_OS_MAC)
-#include "utility_mac.cpp"
-#else
-#include "utility_unix.cpp"
-#endif
+namespace {
+constexpr auto bytes = 1024;
+constexpr auto kilobytes = bytes;
+constexpr auto megabytes = bytes * kilobytes;
+constexpr qint64 gigabytes = bytes * megabytes;
+constexpr qint64 terabytes = bytes * gigabytes;
+}
 
 namespace OCC {
 
@@ -64,14 +64,13 @@ Q_LOGGING_CATEGORY(lcUtility, "nextcloud.sync.utility", QtInfoMsg)
 bool Utility::writeRandomFile(const QString &fname, int size)
 {
     int maxSize = 10 * 10 * 1024;
-    qsrand(QDateTime::currentMSecsSinceEpoch());
 
     if (size == -1)
-        size = qrand() % maxSize;
+        size = rand() % maxSize;
 
     QString randString;
     for (int i = 0; i < size; i++) {
-        int r = qrand() % 128;
+        int r = rand() % 128;
         randString.append(QChar(r));
     }
 
@@ -104,51 +103,41 @@ QString Utility::formatFingerprint(const QByteArray &fmhash, bool colonSeparated
     return fp;
 }
 
-void Utility::setupFavLink(const QString &folder)
+
+QString Utility::octetsToString(const qint64 octets)
 {
-    setupFavLink_private(folder);
-}
+    auto unitName = QCoreApplication::translate("Utility", "%L1 B");
+    qreal dataSize = octets;
 
-QString Utility::octetsToString(qint64 octets)
-{
-#define THE_FACTOR 1024
-    static const qint64 kb = THE_FACTOR;
-    static const qint64 mb = THE_FACTOR * kb;
-    static const qint64 gb = THE_FACTOR * mb;
+    // Display decimals when value < TB and unit < 10
+    auto showDecimals = true;
 
-    QString s;
-    qreal value = octets;
-
-    // Whether we care about decimals: only for GB/MB and only
-    // if it's less than 10 units.
-    bool round = true;
-
-    // do not display terra byte with the current units, as when
-    // the MB, GB and KB units were made, there was no TB,
-    // see the JEDEC standard
-    // https://en.wikipedia.org/wiki/JEDEC_memory_standards
-    if (octets >= gb) {
-        s = QCoreApplication::translate("Utility", "%L1 GB");
-        value /= gb;
-        round = false;
-    } else if (octets >= mb) {
-        s = QCoreApplication::translate("Utility", "%L1 MB");
-        value /= mb;
-        round = false;
-    } else if (octets >= kb) {
-        s = QCoreApplication::translate("Utility", "%L1 KB");
-        value /= kb;
-    } else {
-        s = QCoreApplication::translate("Utility", "%L1 B");
+    if (octets >= terabytes) {
+        unitName = QCoreApplication::translate("Utility", "%L1 TB");
+        dataSize /= terabytes;
+        showDecimals = false;
+    } else if (octets >= gigabytes) {
+        unitName = QCoreApplication::translate("Utility", "%L1 GB");
+        dataSize /= gigabytes;
+        showDecimals = false;
+    } else if (octets >= megabytes) {
+        unitName = QCoreApplication::translate("Utility", "%L1 MB");
+        dataSize /= megabytes;
+        showDecimals = false;
+    } else if (octets >= kilobytes) {
+        unitName = QCoreApplication::translate("Utility", "%L1 KB");
+        dataSize /= kilobytes;
     }
 
-    if (value > 9.95)
-        round = true;
+    if (dataSize > 9.95) {
+        showDecimals = true;
+    }
 
-    if (round)
-        return s.arg(qRound(value));
+    if (showDecimals) {
+        return unitName.arg(qRound(dataSize));
+    }
 
-    return s.arg(value, 0, 'g', 2);
+    return unitName.arg(dataSize, 0, 'g', 2);
 }
 
 // Qtified version of get_platforms() in csync_owncloud.c
@@ -195,26 +184,6 @@ QByteArray Utility::friendlyUserAgentString()
     return userAgent.toUtf8();
 }
 
-bool Utility::hasSystemLaunchOnStartup(const QString &appName)
-{
-#if defined(Q_OS_WIN)
-    return hasSystemLaunchOnStartup_private(appName);
-#else
-    Q_UNUSED(appName)
-    return false;
-#endif
-}
-
-bool Utility::hasLaunchOnStartup(const QString &appName)
-{
-    return hasLaunchOnStartup_private(appName);
-}
-
-void Utility::setLaunchOnStartup(const QString &appName, const QString &guiName, bool enable)
-{
-    setLaunchOnStartup_private(appName, guiName, enable);
-}
-
 qint64 Utility::freeDiskSpace(const QString &path)
 {
 #if defined(Q_OS_MAC) || defined(Q_OS_FREEBSD) || defined(Q_OS_FREEBSD_KERNEL) || defined(Q_OS_NETBSD) || defined(Q_OS_OPENBSD)
@@ -223,7 +192,7 @@ qint64 Utility::freeDiskSpace(const QString &path)
         return (qint64)stat.f_bavail * stat.f_frsize;
     }
 #elif defined(Q_OS_UNIX)
-    struct statvfs64 stat;
+    struct statvfs64 stat{};
     if (statvfs64(path.toLocal8Bit().data(), &stat) == 0) {
         return (qint64)stat.f_bavail * stat.f_frsize;
     }
@@ -259,6 +228,11 @@ QString Utility::escape(const QString &in)
     return in.toHtmlEscaped();
 }
 
+int Utility::rand()
+{
+    return QRandomGenerator::global()->bounded(0, RAND_MAX);
+}
+
 void Utility::sleep(int sec)
 {
     QThread::sleep(sec);
@@ -269,7 +243,7 @@ void Utility::usleep(int usec)
     QThread::usleep(usec);
 }
 
-// This can be overriden from the tests
+// This can be overridden from the tests
 OCSYNC_EXPORT bool fsCasePreserving_override = []() -> bool {
     QByteArray env = qgetenv("OWNCLOUD_TEST_CASE_PRESERVING");
     if (!env.isEmpty())
@@ -288,7 +262,7 @@ bool Utility::fileNamesEqual(const QString &fn1, const QString &fn2)
     const QDir fd2(fn2);
 
     // Attention: If the path does not exist, canonicalPath returns ""
-    // ONLY use this function with existing pathes.
+    // ONLY use this function with existing paths.
     const QString a = fd1.canonicalPath();
     const QString b = fd2.canonicalPath();
     bool re = !a.isEmpty() && QString::compare(a, b, fsCasePreserving() ? Qt::CaseInsensitive : Qt::CaseSensitive) == 0;
@@ -311,7 +285,7 @@ namespace {
         const char *name;
         quint64 msec;
 
-        QString description(quint64 value) const
+        [[nodiscard]] QString description(quint64 value) const
         {
             return QCoreApplication::translate("Utility", name, nullptr, value);
         }
@@ -394,12 +368,6 @@ QByteArray Utility::normalizeEtag(QByteArray etag)
     return etag;
 }
 
-bool Utility::hasDarkSystray()
-{
-    return hasDarkSystray_private();
-}
-
-
 QString Utility::platformName()
 {
     return QSysInfo::prettyProductName();
@@ -468,10 +436,8 @@ QString Utility::timeAgoInWords(const QDateTime &dt, const QDateTime &from)
         now = from;
     }
 
-    if (dt.daysTo(now) == 1) {
-        return QObject::tr("%n day ago", "", dt.daysTo(now));
-    } else if (dt.daysTo(now) > 1) {
-        return QObject::tr("%n days ago", "", dt.daysTo(now));
+    if (dt.daysTo(now) >= 1) {
+        return QObject::tr("%nd", "delay in days after an activity", dt.daysTo(now));
     } else {
         qint64 secs = dt.secsTo(now);
         if (secs < 0) {
@@ -480,11 +446,7 @@ QString Utility::timeAgoInWords(const QDateTime &dt, const QDateTime &from)
 
         if (floor(secs / 3600.0) > 0) {
             int hours = floor(secs / 3600.0);
-            if (hours == 1) {
-                return (QObject::tr("%n hour ago", "", hours));
-            } else {
-                return (QObject::tr("%n hours ago", "", hours));
-            }
+            return (QObject::tr("%nh", "delay in hours after an activity", hours));
         } else {
             int minutes = qRound(secs / 60.0);
 
@@ -492,13 +454,10 @@ QString Utility::timeAgoInWords(const QDateTime &dt, const QDateTime &from)
                 if (secs < 5) {
                     return QObject::tr("now");
                 } else {
-                    return QObject::tr("Less than a minute ago");
+                    return QObject::tr("1m", "one minute after activity date and time");
                 }
-
-            } else if (minutes == 1) {
-                return (QObject::tr("%n minute ago", "", minutes));
             } else {
-                return (QObject::tr("%n minutes ago", "", minutes));
+                return (QObject::tr("%nm", "delay in minutes after an activity", minutes));
             }
         }
     }
@@ -614,35 +573,21 @@ QString Utility::makeConflictFileName(
     return conflictFileName;
 }
 
-bool Utility::isConflictFile(const char *name)
-{
-    const char *bname = std::strrchr(name, '/');
-    if (bname) {
-        bname += 1;
-    } else {
-        bname = name;
-    }
-
-    // Old pattern
-    if (std::strstr(bname, "_conflict-"))
-        return true;
-
-    // New pattern
-    if (std::strstr(bname, "(conflicted copy"))
-        return true;
-
-    return false;
-}
-
 bool Utility::isConflictFile(const QString &name)
 {
     auto bname = name.midRef(name.lastIndexOf(QLatin1Char('/')) + 1);
 
-    if (bname.contains(QStringLiteral("_conflict-")))
+    if (bname.contains(QStringLiteral("_conflict-"))) {
         return true;
+    }
 
-    if (bname.contains(QStringLiteral("(conflicted copy")))
+    if (bname.contains(QStringLiteral("(conflicted copy"))) {
         return true;
+    }
+
+    if (isCaseClashConflictFile(name)) {
+        return true;
+    }
 
     return false;
 }
@@ -686,7 +631,7 @@ bool Utility::isPathWindowsDrivePartitionRoot(const QString &path)
         return false;
     }
 
-    // must mutch a pattern "[A-Za-z]:"
+    // must match a pattern "[A-Za-z]:"
     if (!(path.at(1) == QLatin1Char(':') && path.at(0).isLetter())) {
         return false;
     }
@@ -712,4 +657,41 @@ QString Utility::sanitizeForFileName(const QString &name)
     return result;
 }
 
+QString Utility::makeCaseClashConflictFileName(const QString &filename, const QDateTime &datetime)
+{
+    auto conflictFileName(filename);
+    // Add conflict tag before the extension.
+    auto dotLocation = conflictFileName.lastIndexOf(QLatin1Char('.'));
+    // If no extension, add it at the end  (take care of cases like foo/.hidden or foo.bar/file)
+    if (dotLocation <= conflictFileName.lastIndexOf(QLatin1Char('/')) + 1) {
+        dotLocation = conflictFileName.size();
+    }
+
+    auto conflictMarker = QStringLiteral(" (case clash from ");
+    conflictMarker += datetime.toString(QStringLiteral("yyyy-MM-dd hhmmss")) + QLatin1Char(')');
+
+    conflictFileName.insert(dotLocation, conflictMarker);
+    return conflictFileName;
+}
+
+bool Utility::isCaseClashConflictFile(const QString &name)
+{
+    const auto bname = name.midRef(name.lastIndexOf(QLatin1Char('/')) + 1);
+
+    return bname.contains(QStringLiteral("(case clash from"));
+}
+
+QString Utility::trailingSlashPath(const QString &path)
+{
+    static const auto slash = QLatin1Char('/');
+    return path.endsWith(slash) ? path : QString(path + slash);
+}
+
+QString Utility::noLeadingSlashPath(const QString &path)
+{
+    static const auto slash = QLatin1Char('/');
+    return path.startsWith(slash) ? path.mid(1) : path;
+}
+
+
 } // namespace OCC
