@@ -18,7 +18,6 @@
 #include "theme.h"
 #include "configfile.h"
 #include "application.h"
-#include "configfile.h"
 #include "owncloudsetupwizard.h"
 #include "accountmanager.h"
 #include "guiutility.h"
@@ -45,7 +44,7 @@
 #include <QScopedValueRollback>
 #include <QMessageBox>
 
-#include <private/qzipwriter_p.h>
+#include <KZip>
 
 #define QTLEGACY (QT_VERSION < QT_VERSION_CHECK(5,9,0))
 
@@ -81,7 +80,7 @@ ZipEntry syncFolderToZipEntry(OCC::Folder *f)
     return fileInfoToZipEntry(journalInfo);
 }
 
-QVector<ZipEntry> createFileList()
+QVector<ZipEntry> createDebugArchiveFileList()
 {
     auto list = QVector<ZipEntry>();
     OCC::ConfigFile cfg;
@@ -91,8 +90,6 @@ QVector<ZipEntry> createFileList()
     const auto logger = OCC::Logger::instance();
 
     if (!logger->logDir().isEmpty()) {
-        list.append({QString(), QStringLiteral("logs")});
-
         QDir dir(logger->logDir());
         const auto infoList = dir.entryInfoList(QDir::Files);
         std::transform(std::cbegin(infoList), std::cend(infoList),
@@ -112,25 +109,24 @@ QVector<ZipEntry> createFileList()
 
 void createDebugArchive(const QString &filename)
 {
-    const auto entries = createFileList();
+    const auto entries = createDebugArchiveFileList();
+
+    KZip zip(filename);
+    zip.open(QIODevice::WriteOnly);
 
-    QZipWriter zip(filename);
     for (const auto &entry : entries) {
-        if (entry.localFilename.isEmpty()) {
-            zip.addDirectory(entry.zipFilename);
-        } else {
-            QFile file(entry.localFilename);
-            if (!file.open(QFile::ReadOnly)) {
-                continue;
-            }
-            zip.addFile(entry.zipFilename, &file);
-        }
+        zip.addLocalFile(entry.localFilename, entry.zipFilename);
     }
 
-    zip.addFile("__nextcloud_client_parameters.txt", QCoreApplication::arguments().join(' ').toUtf8());
+    const auto clientParameters = QCoreApplication::arguments().join(' ').toUtf8();
+    zip.prepareWriting("__nextcloud_client_parameters.txt", {}, {}, clientParameters.size());
+    zip.writeData(clientParameters, clientParameters.size());
+    zip.finishWriting(clientParameters.size());
 
-    const auto buildInfo = QString(OCC::Theme::instance()->about() + "\n\n" + OCC::Theme::instance()->aboutDetails());
-    zip.addFile("__nextcloud_client_buildinfo.txt", buildInfo.toUtf8());
+    const auto buildInfo = QString(OCC::Theme::instance()->aboutInfo() + "\n\n" + OCC::Theme::instance()->aboutDetails()).toUtf8();
+    zip.prepareWriting("__nextcloud_client_buildinfo.txt", {}, {}, buildInfo.size());
+    zip.writeData(buildInfo, buildInfo.size());
+    zip.finishWriting(buildInfo.size());
 }
 }
 
@@ -159,27 +155,27 @@ GeneralSettings::GeneralSettings(QWidget *parent)
     _ui->showInExplorerNavigationPaneCheckBox->setText(txt);
 #endif
 
-    if(Utility::hasSystemLaunchOnStartup(Theme::instance()->appName())) {
-        _ui->autostartCheckBox->setChecked(true);
-        _ui->autostartCheckBox->setDisabled(true);
+    if(const auto hasSystemAutoStart = Utility::hasSystemLaunchOnStartup(Theme::instance()->appName())) {
+        _ui->autostartCheckBox->setChecked(hasSystemAutoStart);
+        _ui->autostartCheckBox->setDisabled(hasSystemAutoStart);
         _ui->autostartCheckBox->setToolTip(tr("You cannot disable autostart because system-wide autostart is enabled."));
-    } else {
-        const bool hasAutoStart = Utility::hasLaunchOnStartup(Theme::instance()->appName());
-        // make sure the binary location is correctly set
-        slotToggleLaunchOnStartup(hasAutoStart);
-        _ui->autostartCheckBox->setChecked(hasAutoStart);
+    } else {       
         connect(_ui->autostartCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::slotToggleLaunchOnStartup);
+        _ui->autostartCheckBox->setChecked(ConfigFile().launchOnSystemStartup());
     }
 
     // setup about section
-    QString about = Theme::instance()->about();
-    _ui->aboutLabel->setTextInteractionFlags(Qt::TextSelectableByMouse | Qt::TextBrowserInteraction);
-    _ui->aboutLabel->setText(about);
-    _ui->aboutLabel->setOpenExternalLinks(true);
+    _ui->infoAndUpdatesLabel->setTextInteractionFlags(Qt::TextSelectableByMouse | Qt::TextBrowserInteraction);
+    _ui->infoAndUpdatesLabel->setText(Theme::instance()->about());
+    _ui->infoAndUpdatesLabel->setOpenExternalLinks(true);
 
     // About legal notice
     connect(_ui->legalNoticeButton, &QPushButton::clicked, this, &GeneralSettings::slotShowLegalNotice);
 
+    connect(_ui->usageDocumentationButton, &QPushButton::clicked, this, []() {
+        Utility::openBrowser(QUrl(Theme::instance()->helpUrl()));
+    });
+
     loadMiscSettings();
     // updater info now set in: customizeStyle
     //slotUpdateInfo();
@@ -189,7 +185,10 @@ GeneralSettings::GeneralSettings(QWidget *parent)
     connect(_ui->crashreporterCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
     connect(_ui->newFolderLimitCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
     connect(_ui->newFolderLimitSpinBox, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), this, &GeneralSettings::saveMiscSettings);
+    connect(_ui->existingFolderLimitCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
+    connect(_ui->stopExistingFolderNowBigSyncCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
     connect(_ui->newExternalStorage, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
+    connect(_ui->moveFilesToTrashCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
 
 #ifndef WITH_CRASHREPORTER
     _ui->crashreporterCheckBox->setVisible(false);
@@ -249,15 +248,24 @@ void GeneralSettings::loadMiscSettings()
 {
     QScopedValueRollback<bool> scope(_currentlyLoading, true);
     ConfigFile cfgFile;
+
     _ui->monoIconsCheckBox->setChecked(cfgFile.monoIcons());
     _ui->serverNotificationsCheckBox->setChecked(cfgFile.optionalServerNotifications());
     _ui->callNotificationsCheckBox->setEnabled(_ui->serverNotificationsCheckBox->isEnabled());
     _ui->callNotificationsCheckBox->setChecked(cfgFile.showCallNotifications());
     _ui->showInExplorerNavigationPaneCheckBox->setChecked(cfgFile.showInExplorerNavigationPane());
     _ui->crashreporterCheckBox->setChecked(cfgFile.crashReporter());
+    _ui->newExternalStorage->setChecked(cfgFile.confirmExternalStorage());
+    _ui->monoIconsCheckBox->setChecked(cfgFile.monoIcons());
+    _ui->moveFilesToTrashCheckBox->setChecked(cfgFile.moveToTrash());
+
     auto newFolderLimit = cfgFile.newBigFolderSizeLimit();
     _ui->newFolderLimitCheckBox->setChecked(newFolderLimit.first);
     _ui->newFolderLimitSpinBox->setValue(newFolderLimit.second);
+    _ui->existingFolderLimitCheckBox->setEnabled(_ui->newFolderLimitCheckBox->isChecked());
+    _ui->existingFolderLimitCheckBox->setChecked(_ui->newFolderLimitCheckBox->isChecked() && cfgFile.notifyExistingFoldersOverLimit());
+    _ui->stopExistingFolderNowBigSyncCheckBox->setEnabled(_ui->existingFolderLimitCheckBox->isChecked());
+    _ui->stopExistingFolderNowBigSyncCheckBox->setChecked(_ui->existingFolderLimitCheckBox->isChecked() && cfgFile.stopSyncingExistingFoldersOverLimit());
     _ui->newExternalStorage->setChecked(cfgFile.confirmExternalStorage());
     _ui->monoIconsCheckBox->setChecked(cfgFile.monoIcons());
 }
@@ -265,20 +273,31 @@ void GeneralSettings::loadMiscSettings()
 #if defined(BUILD_UPDATER)
 void GeneralSettings::slotUpdateInfo()
 {
-    if (ConfigFile().skipUpdateCheck() || !Updater::instance()) {
+    const auto updater = Updater::instance();
+    if (ConfigFile().skipUpdateCheck() || !updater) {
         // updater disabled on compile
-        _ui->updatesGroupBox->setVisible(false);
+        _ui->updatesContainer->setVisible(false);
         return;
     }
 
+    if (updater) {
+        connect(_ui->updateButton,
+                &QAbstractButton::clicked,
+                this,
+
+                &GeneralSettings::slotUpdateCheckNow,
+                Qt::UniqueConnection);
+        connect(_ui->autoCheckForUpdatesCheckBox, &QAbstractButton::toggled, this,
+                &GeneralSettings::slotToggleAutoUpdateCheck, Qt::UniqueConnection);
+        _ui->autoCheckForUpdatesCheckBox->setChecked(ConfigFile().autoUpdateCheck());
+    }
+
     // Note: the sparkle-updater is not an OCUpdater
-    auto *ocupdater = qobject_cast<OCUpdater *>(Updater::instance());
+    auto *ocupdater = qobject_cast<OCUpdater *>(updater);
     if (ocupdater) {
         connect(ocupdater, &OCUpdater::downloadStateChanged, this, &GeneralSettings::slotUpdateInfo, Qt::UniqueConnection);
         connect(_ui->restartButton, &QAbstractButton::clicked, ocupdater, &OCUpdater::slotStartInstaller, Qt::UniqueConnection);
         connect(_ui->restartButton, &QAbstractButton::clicked, qApp, &QApplication::quit, Qt::UniqueConnection);
-        connect(_ui->updateButton, &QAbstractButton::clicked, this, &GeneralSettings::slotUpdateCheckNow, Qt::UniqueConnection);
-        connect(_ui->autoCheckForUpdatesCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::slotToggleAutoUpdateCheck);
 
         QString status = ocupdater->statusString(OCUpdater::UpdateStatusStringFormat::Html);
         Theme::replaceLinkColorStringBackgroundAware(status);
@@ -294,14 +313,17 @@ void GeneralSettings::slotUpdateInfo()
         _ui->updateButton->setEnabled(ocupdater->downloadState() != OCUpdater::CheckingServer &&
                                       ocupdater->downloadState() != OCUpdater::Downloading &&
                                       ocupdater->downloadState() != OCUpdater::DownloadComplete);
-
-        _ui->autoCheckForUpdatesCheckBox->setChecked(ConfigFile().autoUpdateCheck());
     }
 #if defined(Q_OS_MAC) && defined(HAVE_SPARKLE)
-    else if (auto sparkleUpdater = qobject_cast<SparkleUpdater *>(Updater::instance())) {
+    else if (const auto sparkleUpdater = qobject_cast<SparkleUpdater *>(updater)) {
+        connect(sparkleUpdater, &SparkleUpdater::statusChanged, this, &GeneralSettings::slotUpdateInfo, Qt::UniqueConnection);
         _ui->updateStateLabel->setText(sparkleUpdater->statusString());
         _ui->restartButton->setVisible(false);
-        connect(_ui->updateButton, &QAbstractButton::clicked, this, &GeneralSettings::slotUpdateCheckNow, Qt::UniqueConnection);
+
+        const auto updaterState = sparkleUpdater->state();
+        const auto enableUpdateButton = updaterState == SparkleUpdater::State::Idle ||
+                                        updaterState == SparkleUpdater::State::Unknown;
+        _ui->updateButton->setEnabled(enableUpdateButton);
     }
 #endif
 
@@ -311,7 +333,7 @@ void GeneralSettings::slotUpdateInfo()
         this, &GeneralSettings::slotUpdateChannelChanged, Qt::UniqueConnection);
 }
 
-void GeneralSettings::slotUpdateChannelChanged(const QString &translatedChannel)
+void GeneralSettings::slotUpdateChannelChanged()
 {
     const auto updateChannelToLocalized = [](const QString &channel) {
         auto decodedTranslatedChannel = QString{};
@@ -326,26 +348,15 @@ void GeneralSettings::slotUpdateChannelChanged(const QString &translatedChannel)
     };
 
     const auto updateChannelFromLocalized = [](const int index) {
-        auto channel = QString{};
-
-        switch (index)
-        {
-        case 0:
-            channel = QStringLiteral("stable");
-            break;
-        case 1:
-            channel = QStringLiteral("beta");
-            break;
-        default:
-            channel = QString{};
+        if (index == 1) {
+            return QStringLiteral("beta");
         }
 
-        return channel;
+        return QStringLiteral("stable");
     };
 
     const auto channel = updateChannelFromLocalized(_ui->updateChannel->currentIndex());
-
-    if (translatedChannel == ConfigFile().updateChannel()) {
+    if (channel == ConfigFile().updateChannel()) {
         return;
     }
 
@@ -416,22 +427,39 @@ void GeneralSettings::slotToggleAutoUpdateCheck()
 
 void GeneralSettings::saveMiscSettings()
 {
-    if (_currentlyLoading)
+    if (_currentlyLoading) {
         return;
-    ConfigFile cfgFile;
-    bool isChecked = _ui->monoIconsCheckBox->isChecked();
-    cfgFile.setMonoIcons(isChecked);
-    Theme::instance()->setSystrayUseMonoIcons(isChecked);
-    cfgFile.setCrashReporter(_ui->crashreporterCheckBox->isChecked());
+    }
 
-    cfgFile.setNewBigFolderSizeLimit(_ui->newFolderLimitCheckBox->isChecked(),
-        _ui->newFolderLimitSpinBox->value());
+    ConfigFile cfgFile;
+
+    const auto useMonoIcons = _ui->monoIconsCheckBox->isChecked();
+    const auto newFolderLimitEnabled = _ui->newFolderLimitCheckBox->isChecked();
+    const auto existingFolderLimitEnabled = newFolderLimitEnabled && _ui->existingFolderLimitCheckBox->isChecked();
+    const auto stopSyncingExistingFoldersOverLimit = existingFolderLimitEnabled && _ui->stopExistingFolderNowBigSyncCheckBox->isChecked();
+    Theme::instance()->setSystrayUseMonoIcons(useMonoIcons);
+
+    cfgFile.setMonoIcons(useMonoIcons);
+    cfgFile.setCrashReporter(_ui->crashreporterCheckBox->isChecked());
+    cfgFile.setMoveToTrash(_ui->moveFilesToTrashCheckBox->isChecked());
+    cfgFile.setNewBigFolderSizeLimit(newFolderLimitEnabled, _ui->newFolderLimitSpinBox->value());
     cfgFile.setConfirmExternalStorage(_ui->newExternalStorage->isChecked());
+    cfgFile.setNotifyExistingFoldersOverLimit(existingFolderLimitEnabled);
+    cfgFile.setStopSyncingExistingFoldersOverLimit(stopSyncingExistingFoldersOverLimit);
+
+    _ui->existingFolderLimitCheckBox->setEnabled(newFolderLimitEnabled);
+    _ui->stopExistingFolderNowBigSyncCheckBox->setEnabled(existingFolderLimitEnabled);
 }
 
 void GeneralSettings::slotToggleLaunchOnStartup(bool enable)
 {
-    Theme *theme = Theme::instance();
+    const auto theme = Theme::instance();
+    if (enable == Utility::hasLaunchOnStartup(theme->appName())) {
+        return;
+    }
+
+    ConfigFile configFile;
+    configFile.setLaunchOnSystemStartup(enable);
     Utility::setLaunchOnStartup(theme->appName(), theme->appNameGUI(), enable);
 }
 
@@ -494,15 +522,18 @@ void GeneralSettings::slotStyleChanged()
 void GeneralSettings::customizeStyle()
 {
     // setup about section
-    QString about = Theme::instance()->about();
-    Theme::replaceLinkColorStringBackgroundAware(about);
-    _ui->aboutLabel->setText(about);
+    const auto aboutText = []() {
+        auto aboutText = Theme::instance()->about();
+        Theme::replaceLinkColorStringBackgroundAware(aboutText);
+        return aboutText;
+    }();
+    _ui->infoAndUpdatesLabel->setText(aboutText);
 
 #if defined(BUILD_UPDATER)
     // updater info
     slotUpdateInfo();
 #else
-    _ui->updatesGroupBox->setVisible(false);
+    _ui->updatesContainer->setVisible(false);
 #endif
 }
 
