@@ -198,6 +198,15 @@ bool Capabilities::hasActivities() const
     return _capabilities.contains("activity");
 }
 
+bool Capabilities::isClientStatusReportingEnabled() const
+{
+    if (!_capabilities.contains(QStringLiteral("security_guard"))) {
+        return false;
+    }
+    const auto securityGuardCaps = _capabilities[QStringLiteral("security_guard")].toMap();
+    return securityGuardCaps.contains(QStringLiteral("diagnostics")) && securityGuardCaps[QStringLiteral("diagnostics")].toBool();
+}
+
 QList<QByteArray> Capabilities::supportedChecksumTypes() const
 {
     QList<QByteArray> list;
@@ -245,6 +254,11 @@ bool Capabilities::filesLockAvailable() const
     return _capabilities["files"].toMap()["locking"].toByteArray() >= "1.0";
 }
 
+bool Capabilities::filesLockTypeAvailable() const
+{
+    return _capabilities["files"].toMap()["api-feature-lock-type"].toByteArray() >= "1.0";
+}
+
 bool Capabilities::userStatus() const
 {
     if (!_capabilities.contains("user_status")) {
@@ -350,6 +364,11 @@ bool Capabilities::uploadConflictFiles() const
     return _capabilities[QStringLiteral("uploadConflictFiles")].toBool();
 }
 
+bool Capabilities::groupFoldersAvailable() const
+{
+    return _capabilities[QStringLiteral("groupfolders")].toMap().value(QStringLiteral("hasGroupFolders"), false).toBool();
+}
+
 QStringList Capabilities::blacklistedFiles() const
 {
     return _capabilities["files"].toMap()["blacklisted_files"].toStringList();
